namespace FS.API.UCenter.UCAPI
{
    using FS.API.UCenter;
    using System;
    using System.Text;
    using System.Web;
    /// <summary>
    /// Դ
    /// ޸ߣDozer
    /// ϵʽmail@dozerstudio.com
    /// </summary>
    public abstract class UCAPIBase : IHttpHandler
    {
        protected bool API_DELETEUSER = true;
        protected bool API_GETCREDIT = true;
        protected bool API_GETCREDITSETTINGS = true;
        protected bool API_GETTAG = true;
        protected bool API_RENAMEUSER = true;
        protected string API_RETURN_FAILED = "-1";
        protected string API_RETURN_FORBIDDEN = "-2";
        protected string API_RETURN_SUCCEED = "1";
        protected bool API_SYNLOGIN = true;
        protected bool API_SYNLOGOUT = true;
        protected bool API_UPDATEAPPS = true;
        protected bool API_UPDATEBADWORDS = true;
        protected bool API_UPDATECLIENT = true;
        protected bool API_UPDATECREDIT = true;
        protected bool API_UPDATECREDITSETTINGS = true;
        protected bool API_UPDATEHOSTS = true;
        protected bool API_UPDATEPW = true;
        private bool isPostMethod = false;
        private UCRequestObject ucRequest = null;

        protected UCAPIBase()
        {
        }

        private string __DeleteUser()
        {
            if (!this.API_DELETEUSER)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            if (Utils.IsIntArray(this.Parameter_IDS))
            {
                string[] ids = this.Parameter_IDS.Split(new char[] { ',' });
                if (this.DeleteUser(ids))
                {
                    return this.API_RETURN_SUCCEED;
                }
            }
            return this.API_RETURN_FAILED;
        }

        private string __GetCredit()
        {
            if (!this.API_GETCREDIT)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            int uid = Utils.StringToInt(this.Parameter_UID);
            int creditId = Utils.StringToInt(this.Parameter_CREDIT);
            if (uid <= 0)
            {
                return "0";
            }
            return this.GetCredit(uid, creditId).ToString();
        }

        private string __GetCreditSettings()
        {
            if (!this.API_GETCREDITSETTINGS)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            return this.GetCreditSettings();
        }

        private string __GetTag()
        {
            return "";
        }

        private string __RenameUser()
        {
            if (!this.API_RENAMEUSER)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            int uid = Utils.StringToInt(this.Parameter_UID);
            string oldname = this.Parameter_OLDUSERNAME;
            string str2 = this.Parameter_NEWUSERNAME;
            if (((uid > 0) || !string.IsNullOrEmpty(str2)) && this.RenameUser(uid, oldname, str2))
            {
                return this.API_RETURN_SUCCEED;
            }
            return this.API_RETURN_FAILED;
        }

        private string __Synlogin()
        {
            if (!this.API_SYNLOGIN)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            int uid = Utils.StringToInt(this.Parameter_UID);
            if ((uid > 0) && this.Synlogin(uid))
            {
                return this.API_RETURN_SUCCEED;
            }
            return this.API_RETURN_FAILED;
        }

        private void __Synlogout()
        {
            if (this.API_SYNLOGOUT)
            {
                this.Synlogout();
            }
        }

        private string __UpdateApps()
        {
            if (!this.API_UPDATEAPPS || !this.isPostMethod)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            this.UpdateApps();
            return this.API_RETURN_SUCCEED;
        }

        private string __UpdateBadwords()
        {
            if (!this.API_UPDATEBADWORDS || !this.isPostMethod)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            this.UpdateBadwords();
            return this.API_RETURN_SUCCEED;
        }

        private string __UpdateClient()
        {
            if (!this.API_UPDATECLIENT || !this.isPostMethod)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            this.UpdateClient();
            return this.API_RETURN_SUCCEED;
        }

        private string __UpdateCredit()
        {
            if (!this.API_UPDATECREDIT)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            int creditId = Utils.StringToInt(this.Parameter_CREDIT);
            int amount = Utils.StringToInt(this.Parameter_AMOUNT);
            int uid = Utils.StringToInt(this.Parameter_UID);
            if (this.UpdateCredit(uid, creditId, amount))
            {
                return this.API_RETURN_SUCCEED;
            }
            return this.API_RETURN_FAILED;
        }

        private string __UpdateCreditSettings()
        {
            if (!this.API_UPDATECREDITSETTINGS)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            this.UpdateCreditSettings(this.Parameter_CREDIT);
            return this.API_RETURN_SUCCEED;
        }

        private string __UpdateHosts()
        {
            if (!this.API_UPDATEHOSTS || !this.isPostMethod)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            this.UpdateHosts();
            return this.API_RETURN_SUCCEED;
        }

        private string __UpdatePW()
        {
            if (!this.API_UPDATEPW)
            {
                return this.API_RETURN_FORBIDDEN;
            }
            string str = this.Parameter_USERNAME;
            string str2 = this.Parameter_PASSWORD;
            if ((!string.IsNullOrEmpty(str) && !string.IsNullOrEmpty(str2)) && this.UpdatePW(str, Utils.MD5(str2)))
            {
                return this.API_RETURN_SUCCEED;
            }
            return this.API_RETURN_FAILED;
        }

        public abstract bool DeleteUser(string[] ids);
        public abstract int GetCredit(int uid, int creditId);
        public abstract string GetCreditSettings();
        public string GetRequestParameter(string keyName)
        {
            if (this.ucRequest == null)
            {
                return "";
            }
            return this.ucRequest.Parameters[keyName].Trim();
        }

        public void ProcessRequest(HttpContext context)
        {
            this.ucRequest = new UCRequestObject(context.Request, this.EncryptKeyNameInRequest, UCConfig.UC_KEY, this.UrlDecodeEncryptValue, this.UCRequestEncoding);
            this.isPostMethod = context.Request.HttpMethod == "POST";
            if (this.ucRequest.IsInvalidAction || this.ucRequest.IsInvalidRequest)
            {
                context.Response.Write("Invalid Request");
                context.Response.End();
            }
            if (this.ucRequest.IsAuthracationExpiried)
            {
                context.Response.Write("Authracation has expiried");
                context.Response.End();
            }
            context.Response.Clear();
            switch (this.ucRequest.Action)
            {
                case "test":
                    context.Response.Write(this.API_RETURN_SUCCEED);
                    context.Response.End();
                    return;

                case "addfeed":
                    context.Response.Write(this.API_RETURN_SUCCEED);
                    context.Response.End();
                    return;

                case "deleteuser":
                    context.Response.Write(this.__DeleteUser());
                    context.Response.End();
                    return;

                case "getcreditsettings":
                    context.Response.Write(this.__GetCreditSettings());
                    context.Response.End();
                    return;

                case "gettag":
                    context.Response.Write(this.__GetTag());
                    context.Response.End();
                    return;

                case "renameuser":
                    context.Response.Write(this.__RenameUser());
                    context.Response.End();
                    return;

                case "synlogin":
                    context.Response.Write(this.__Synlogin());
                    context.Response.End();
                    return;

                case "synlogout":
                    context.Response.Headers.Add("P3P", "CP=\"CURa ADMa DEVa PSAo PSDo OUR BUS UNI PUR INT DEM STA PRE COM NAV OTC NOI DSP COR\"");
                    this.__Synlogout();
                    context.Response.End();
                    return;

                case "updateapps":
                    context.Response.Write(this.__UpdateApps());
                    context.Response.End();
                    return;

                case "updatebadwords":
                    context.Response.Write(this.__UpdateBadwords());
                    context.Response.End();
                    return;

                case "updateclient":
                    context.Response.Write(this.__UpdateClient());
                    context.Response.End();
                    return;

                case "updatecredit":
                    context.Response.Write(this.__UpdateCredit());
                    context.Response.End();
                    return;

                case "updatecreditsettings":
                    context.Response.Write(this.__UpdateCreditSettings());
                    context.Response.End();
                    return;

                case "updatehosts":
                    context.Response.Write(this.__UpdateHosts());
                    context.Response.End();
                    return;

                case "updatepw":
                    context.Response.Write(this.__UpdatePW());
                    context.Response.End();
                    return;
            }
        }

        public abstract bool RenameUser(int uid, string oldname, string newname);
        public abstract bool Synlogin(int uid);
        public abstract void Synlogout();
        public abstract void UpdateApps();
        public abstract void UpdateBadwords();
        public abstract void UpdateClient();
        public abstract bool UpdateCredit(int uid, int creditId, int amount);
        public abstract void UpdateCreditSettings(string creditSettings);
        public abstract void UpdateHosts();
        public abstract bool UpdatePW(string uname, string pwd);

        public virtual string EncryptKeyNameInRequest
        {
            get
            {
                return "code";
            }
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }

        protected virtual string Parameter_AMOUNT
        {
            get
            {
                return this.GetRequestParameter("amount");
            }
        }

        protected virtual string Parameter_CREDIT
        {
            get
            {
                return this.GetRequestParameter("credit");
            }
        }

        protected virtual string Parameter_IDS
        {
            get
            {
                return this.GetRequestParameter("ids");
            }
        }

        protected virtual string Parameter_NEWUSERNAME
        {
            get
            {
                return this.GetRequestParameter("newusername");
            }
        }

        protected virtual string Parameter_OLDUSERNAME
        {
            get
            {
                return this.GetRequestParameter("oldusername");
            }
        }

        protected virtual string Parameter_PASSWORD
        {
            get
            {
                return this.GetRequestParameter("password");
            }
        }

        protected virtual string Parameter_UID
        {
            get
            {
                return this.GetRequestParameter("uid");
            }
        }

        protected virtual string Parameter_USERNAME
        {
            get
            {
                return this.GetRequestParameter("username");
            }
        }

        public virtual Encoding UCRequestEncoding
        {
            get
            {
                return Encoding.Default;
            }
        }

        public virtual bool UrlDecodeEncryptValue
        {
            get
            {
                return false;
            }
        }
    }
}

