namespace FS.API.UCenter.UCAPI
{
    using FS.API.UCenter;
    using System;
    using System.Collections.Specialized;
    using System.Text;
    using System.Web;
    /// <summary>
    /// 来源：网络
    /// 修改者：Dozer
    /// 联系方式：mail@dozerstudio.com
    /// </summary>
    internal class UCRequestObject
    {
        private string action;
        private bool isAuthracationExpiried;
        private bool isInvalidRequest;
        private StringBuilder logMsg;
        private NameValueCollection parameters;
        private int timestamp;

        public UCRequestObject(HttpRequest uc_request, string encryptKeyName, string uc_key) : this(uc_request, encryptKeyName, uc_key, true, Encoding.Default)
        {
        }

        public UCRequestObject(HttpRequest uc_request, string encryptKeyName, string uc_key, bool decodeUrl) : this(uc_request, encryptKeyName, uc_key, decodeUrl, Encoding.Default)
        {
        }

        public UCRequestObject(HttpRequest uc_request, string encryptKeyName, string uc_key, bool decodeUrl, Encoding requestEncoding)
        {
            this.logMsg = new StringBuilder();
            this.logMsg.AppendLine(DateTime.Now.ToString());
            this.logMsg.AppendLine("原始请求：" + uc_request.RawUrl);
            this.action = string.Empty;
            this.timestamp = 0;
            this.isInvalidRequest = false;
            this.isAuthracationExpiried = false;
            this.parameters = new NameValueCollection();
            if (uc_request[encryptKeyName] != null)
            {
                string str = uc_request.QueryString[encryptKeyName].Trim();
                this.logMsg.AppendLine("Code参数：" + str);
                if (decodeUrl)
                {
                    str = HttpUtility.UrlDecode(str);
                }
                str = UCAuthCode.AuthCodeDecode(str, uc_key);
                this.logMsg.AppendLine("Code解密：" + str);
                this.logMsg.AppendLine("------------------------\r\n\r\n");
                this.parameters = HttpUtility.ParseQueryString(str, requestEncoding);
            }
            if (this.parameters.Count == 0)
            {
                this.isInvalidRequest = true;
            }
            if (this.parameters["action"] != null)
            {
                this.action = this.parameters["action"].Trim().ToLower();
            }
            if (this.parameters["time"] != null)
            {
                this.timestamp = Utils.StringToInt(this.parameters["time"].Trim().ToLower());
            }
            if ((Utils.UnixTimestamp() - this.timestamp) > 0xe10)
            {
                this.isAuthracationExpiried = true;
            }
            Utils.Log(this.logMsg.ToString());
        }

        public string Action
        {
            get
            {
                return this.action;
            }
        }

        public bool IsAuthracationExpiried
        {
            get
            {
                return this.isAuthracationExpiried;
            }
        }

        public bool IsInvalidAction
        {
            get
            {
                return !UCRequestActions.IsValidAction(this.action);
            }
        }

        public bool IsInvalidRequest
        {
            get
            {
                return this.isInvalidRequest;
            }
        }

        public NameValueCollection Parameters
        {
            get
            {
                return this.parameters;
            }
        }

        public int Timestamp
        {
            get
            {
                return this.timestamp;
            }
        }
    }
}

