namespace FS.API.UCenter.UCClient
{
    using FS.API.UCenter;
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.IO;
    using System.Net;
    using System.Text;
    using System.Web;

    public abstract class UCClientBase : IUCClient
    {
        protected UCClientBase()
        {
        }

        protected virtual string GetFlashPlayerCode(string movie, bool html)
        {
            if (html)
            {
                StringBuilder builder = new StringBuilder();
                builder.AppendLine("<object classid=\"clsid:d27cdb6e-ae6d-11cf-96b8-444553540000\" codebase=\"http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,0,0\" width=\"447\" height=\"477\" id=\"mycamera\" align=\"middle\">");
                builder.AppendLine("<param name=\"allowScriptAccess\" value=\"always\" />");
                builder.AppendLine("<param name=\"scale\" value=\"exactfit\" />");
                builder.AppendLine("<param name=\"wmode\" value=\"transparent\" />");
                builder.AppendLine("<param name=\"quality\" value=\"high\" />");
                builder.AppendLine("<param name=\"bgcolor\" value=\"#ffffff\" />");
                builder.AppendLine("<param name=\"movie\" value=\"{0}\" />");
                builder.AppendLine("<param name=\"menu\" value=\"false\" />");
                builder.AppendLine("<embed src=\"{0}\" quality=\"high\" bgcolor=\"#ffffff\" width=\"447\" height=\"477\" name=\"mycamera\" align=\"middle\" allowScriptAccess=\"always\" allowFullScreen=\"false\" scale=\"exactfit\"  wmode=\"transparent\" type=\"application/x-shockwave-flash\" pluginspage=\"http://www.macromedia.com/go/getflashplayer\" />");
                builder.AppendLine("</object>");
                return string.Format(builder.ToString(), movie);
            }
            return ("{movie:" + movie + "}");
        }


        protected virtual string GetUserAgent()
        {
            if ((HttpContext.Current != null) && (HttpContext.Current.Request != null))
            {
                return HttpContext.Current.Request.ServerVariables["Http_User_Agent"];
            }
            return "Mozilla/4.0(compatible;MSIE6.0;)";
        }

        private void LoopHashtable(Hashtable ht, ref string data, ref string sep)
        {
            foreach (DictionaryEntry entry in ht)
            {
                if (entry.Value is Hashtable)
                {
                    this.LoopHashtable((Hashtable) entry.Value, ref data, ref sep);
                }
                else
                {
                    string str = data;
                    data = str + sep + entry.Key.ToString() + "=" + Utils.PHP_UrlEncode(entry.Value.ToString());
                    sep = "&";
                }
            }
        }

        private void RedirectLocation(string url)
        {
            try
            {
                HttpContext.Current.Response.Clear();
                HttpContext.Current.Response.AppendHeader("Expires", "0");
                HttpContext.Current.Response.AppendHeader("Cache-Control", "private, post-check=0, pre-check=0, max-age=0");
                HttpContext.Current.Response.AppendHeader("Pragma", "no-cache");
                HttpContext.Current.Response.AppendHeader("location", url);
                HttpContext.Current.Response.End();
            }
            catch
            {
            }
        }

        public virtual UCAppCollection UC_App_List()
        {
            return new UCAppCollection(this.UCApiPost("app", "ls", null));
        }

        public virtual string UC_Avatar(int uid, string type, bool html)
        {
            string str = this.UCApiInput(string.Format("uid={0}", uid));
            string movie = string.Format("{0}images/camera.swf?inajax=1&appid={1}&input={2}&agent={3}&ucapi={4}&avatartype={5}", new object[] { UCConfig.UC_API, UCConfig.UC_APPID, str, Utils.MD5(this.GetUserAgent()), Utils.PHP_UrlEncode(UCConfig.UC_API), type });
            return this.GetFlashPlayerCode(movie, html);
        }

        public virtual bool UC_Check_Avatar(int uid, string size, string type)
        {
            string url = string.Format("{0}avatar.php?uid={1}&size={2}&type={3}&check_file_exists=1", new object[] { UCConfig.UC_API, uid, size, type });
            return (this.UCFOpen(url, 0x400, "", "", true, UCConfig.UC_IP, 0x4e20, true) == "1");
        }

        public virtual bool UC_Credit_Exchange_Request(int uid, int from, int to, int toAppId, int amount)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("from", from);
            args.Add("to", to);
            args.Add("toappid", toAppId);
            args.Add("amount", amount);
            return (this.UCApiPost("credit", "request", args) == "1");
        }

        public virtual int UC_Feed_Add(string icon, int uid, string username, string title_template, string title_data, string body_template, string body_data, string body_general, string target_ids, List<UC_Image> images)
        {
            Hashtable args = new Hashtable();
            args.Add("icon", icon);
            args.Add("appid", UCConfig.UC_APPID);
            args.Add("uid", uid);
            args.Add("username", username);
            args.Add("title_template", title_template);
            args.Add("title_data", title_data);
            args.Add("body_template", body_template);
            args.Add("body_data", body_data);
            args.Add("body_general", body_general);
            args.Add("target_ids", target_ids);
            if (images != null)
            {
                for (int i = 0; i < images.Count; i++)
                {
                    args.Add(string.Format("image_{0}", i + 1), images[i].ImageUrl);
                    args.Add(string.Format("image_{0}_link", i + 1), images[i].ImageLink);
                }
            }
            return Utils.StringToInt(this.UCApiPost("feed", "add", args));
        }

        public virtual UCFeedCollection UC_Feed_Get(int limit, bool delete)
        {
            Hashtable args = new Hashtable();
            args.Add("limit", limit);
            args.Add("delete", delete ? "1" : "0");
            return new UCFeedCollection(this.UCApiPost("feed", "get", args));
        }

        public virtual bool UC_Friend_Add(int uid, int friendid, string comment)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("friendid", friendid);
            args.Add("comment", comment);
            return (this.UCApiPost("friend", "add", args) == "1");
        }

        public int UC_Friend_Delete(int uid, string[] friendids)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("friendid", string.Join(",", friendids));
            return Utils.StringToInt(this.UCApiPost("friend", "delete", args));
        }

        public virtual UCFriendCollection UC_Friend_List(int uid, int page, int pagesize, int totalnum, short direction)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("page", page);
            args.Add("pagesize", pagesize);
            args.Add("totalnum", totalnum);
            args.Add("direction", direction);
            return new UCFriendCollection(this.UCApiPost("friend", "ls", args));
        }

        public virtual int UC_Friend_TotalNum(int uid, short direction)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("direction", direction);
            return Utils.StringToInt(this.UCApiPost("friend", "totalnum", args));
        }

        public virtual UC_User UC_Get_User(string username, bool isuid)
        {
            Hashtable args = new Hashtable();
            args.Add("username", username);
            args.Add("isuid", isuid ? "1" : "0");
            return new UC_User(this.UCApiPost("user", "get_user", args));
        }

        public virtual UCMailQueueReturn UC_Mail_Queue(string[] uids, string[] emails, string subject, string message, string frommail, string charset, bool htmlon, int level)
        {
            Hashtable args = new Hashtable();
            args.Add("uids", string.Join(",", uids));
            args.Add("emails", string.Join(",", emails));
            args.Add("subject", subject);
            args.Add("message", message);
            args.Add("frommail", frommail);
            args.Add("charset", string.IsNullOrEmpty(charset) ? UCConfig.UC_CHARSET : charset);
            args.Add("htmlon", htmlon);
            args.Add("level", ((level <= 0) || (level > 0x7f)) ? 1 : level);
            return new UCMailQueueReturn(this.UCApiPost("mail", "add", args));
        }

        public virtual bool UC_PM_Blackls_Add(int uid, string[] usernames)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("username", string.Join(",", usernames));
            return (this.UCApiPost("pm", "blackls_add", args) == "1");
        }

        public virtual void UC_PM_Blackls_Delete(int uid, string[] usernames)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("username", string.Join(",", usernames));
            this.UCApiPost("pm", "blackls_delete", args);
        }

        public virtual string UC_PM_Blackls_Get(int uid)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            return this.UCApiPost("pm", "blackls_get", args);
        }

        public virtual bool UC_PM_Blackls_Set(int uid, string blackls)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("blackls", blackls);
            return (this.UCApiPost("pm", "blackls_set", args) == "1");
        }

        public virtual UC_PM_Check UC_PM_CheckNew(int uid, short more)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("more", more);
            return new UC_PM_Check(this.UCApiPost("pm", "check_newpm", args));
        }

        public virtual int UC_PM_Delete(int uid, string folder, string[] pmids)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("folder", folder);
            args.Add("pmids", string.Join(",", pmids));
            return Utils.StringToInt(this.UCApiPost("pm", "delete", args));
        }

        public virtual int UC_PM_DeleteUser(int uid, string[] toUids)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("touids", string.Join(",", toUids));
            return Utils.StringToInt(this.UCApiPost("pm", "deleteuser", args));
        }

        public virtual void UC_PM_Ignore(int uid)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            this.UCApiPost("pm", "ignore", args);
        }

        public virtual UCPMListReturn UC_PM_List(int uid, int page, int pagesize, string folder, string filter, int msglen)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("page", page);
            args.Add("pagesize", pagesize);
            args.Add("folder", folder);
            args.Add("filter", filter);
            args.Add("msglen", msglen);
            return new UCPMListReturn(this.UCApiPost("pm", "ls", args));
        }

        public virtual void UC_PM_Location(int uid, bool newpm)
        {
            string url = this.UCApiUrl("pm_client", "ls", "uid=" + uid.ToString(), newpm ? "&folder=newbox" : "");
            this.RedirectLocation(url);
        }

        public virtual void UC_PM_ReadStatus(int uid, string[] fromUids, string[] pmids, bool readed)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("uids", string.Join(",", fromUids));
            args.Add("pmids", string.Join(",", pmids));
            args.Add("status", readed ? "0" : "1");
            this.UCApiPost("pm", "readstatus", args);
        }

        public virtual int UC_PM_Send(int fromuid, string msgto, string subject, string message, bool instantly, int replypmid, bool isusername)
        {
            if (instantly)
            {
                Hashtable args = new Hashtable();
                args.Add("fromuid", fromuid);
                args.Add("msgto", msgto);
                args.Add("subject", subject);
                args.Add("message", message);
                args.Add("replypmid", replypmid);
                args.Add("isusername", isusername ? "1" : "0");
                return Utils.StringToInt(this.UCApiPost("pm", "sendpm", args));
            }
            string url = this.UCApiUrl("pm_client", "send", "uid=" + fromuid.ToString() + "", string.Format("&msgto={0}&subject={1}&message={2}{3}", new object[] { Utils.PHP_UrlEncode(msgto), Utils.PHP_UrlEncode(subject), Utils.PHP_UrlEncode(message), (replypmid > 0) ? ("&pmid=" + replypmid.ToString() + "&do=reply") : "" }));
            this.RedirectLocation(url);
            return -999;
        }

        public virtual UCPMCollection UC_PM_View(int uid, int pmid, int toUid, short daterange)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("pmid", pmid);
            args.Add("touid", toUid);
            args.Add("daterange", daterange);
            return new UCPMCollection(this.UCApiPost("pm", "view", args));
        }

        public virtual UC_PM UC_PM_ViewNode(int uid, short type, int pmid)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            args.Add("pmid", pmid);
            args.Add("type", type);
            return new UC_PM(this.UCApiPost("pm", "viewnode", args));
        }

        public virtual UC_Tag UC_Tag_Get(string tagname, Hashtable nums)
        {
            Hashtable args = new Hashtable();
            args.Add("tagname", tagname);
            args.Add("nums", nums);
            return new UC_Tag(this.UCApiPost("tag", "gettag", args));
        }

        public virtual int UC_User_AddProtected(string[] usernames, string admin)
        {
            Hashtable args = new Hashtable();
            args.Add("username", string.Join(",", usernames));
            args.Add("admin", admin);
            return Utils.StringToInt(this.UCApiPost("user", "addprotected", args));
        }

        public virtual int UC_User_CheckEmail(string email)
        {
            Hashtable args = new Hashtable();
            args.Add("email", email);
            return Utils.StringToInt(this.UCApiPost("user", "check_email", args));
        }

        public virtual int UC_User_CheckName(string username)
        {
            Hashtable args = new Hashtable();
            args.Add("username", username);
            return Utils.StringToInt(this.UCApiPost("user", "check_username", args));
        }

        public virtual int UC_User_Delete(string[] uid)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", string.Join(",", uid));
            return Utils.StringToInt(this.UCApiPost("user", "delete", args));
        }

        public virtual void UC_User_DeleteAvatar(string[] uid)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", string.Join(",", uid));
            this.UCApiPost("user", "deleteavatar", args);
        }

        public virtual int UC_User_DeleteProtected(string[] usernames)
        {
            Hashtable args = new Hashtable();
            args.Add("username", string.Join(",", usernames));
            return Utils.StringToInt(this.UCApiPost("user", "deleteprotected", args));
        }

        public virtual int UC_User_Edit(string username, string oldpw, string newpw, string email, bool ignoreoldpw, int questionid, string answer)
        {
            Hashtable args = new Hashtable();
            args.Add("username", username);
            args.Add("oldpw", oldpw);
            args.Add("newpw", newpw);
            args.Add("email", email);
            args.Add("ignoreoldpw", ignoreoldpw ? "1" : "0");
            args.Add("questionid", questionid);
            args.Add("answer", answer);
            return Utils.StringToInt(this.UCApiPost("user", "edit", args));
        }

        public virtual int UC_User_GetCredit(int appid, int uid, int credit)
        {
            Hashtable args = new Hashtable();
            args.Add("appid", appid);
            args.Add("uid", uid);
            args.Add("credit", credit);
            return Utils.StringToInt(this.UCApiPost("user", "getcredit", args));
        }

        public virtual UCUserCollection UC_User_GetProtected()
        {
            Hashtable args = new Hashtable();
            args.Add("1", "1");
            return new UCUserCollection(this.UCApiPost("user", "getprotected", args));
        }

        public virtual UCLoginReturn UC_User_Login(string username, string password, bool isuid, bool checkques, int questionid, string answer)
        {
            Hashtable args = new Hashtable();
            args.Add("username", username);
            args.Add("password", password);
            args.Add("isuid", isuid ? "1" : "0");
            args.Add("checkques", checkques ? "1" : "0");
            args.Add("questionid", questionid);
            args.Add("answer", answer);
            return new UCLoginReturn(this.UCApiPost("user", "login", args));
        }

        public virtual int UC_User_Merge(string oldusername, string newusername, int uid, string password, string email)
        {
            Hashtable args = new Hashtable();
            args.Add("oldusername", oldusername);
            args.Add("newusername", newusername);
            args.Add("uid", uid);
            args.Add("password", password);
            args.Add("email", email);
            return Utils.StringToInt(this.UCApiPost("user", "merge", args));
        }

        public virtual void UC_User_Merge_Remove(string username)
        {
            Hashtable args = new Hashtable();
            args.Add("username", username);
            this.UCApiPost("user", "merge_remove", args);
        }

        public virtual int UC_User_Register(string username, string password, string email, int questionid, string answer)
        {
            Hashtable args = new Hashtable();
            args.Add("username", username);
            args.Add("password", password);
            args.Add("email", email);
            args.Add("questionid", questionid);
            args.Add("answer", answer);
            return Utils.StringToInt(this.UCApiPost("user", "register", args));
        }

        public virtual string UC_User_Synlogin(int uid)
        {
            Hashtable args = new Hashtable();
            args.Add("uid", uid);
            return this.UCApiPost("user", "synlogin", args);
        }

        public virtual string UC_User_Synlogout()
        {
            return this.UCApiPost("user", "synlogout", new Hashtable());
        }

        private string UCApiInput(string args)
        {
            return Utils.PHP_UrlEncode(UCAuthCode.AuthCodeEncode(args + "&agent=" + Utils.MD5(this.GetUserAgent()) + "&time=" + Utils.PHP_Time().ToString(), UCConfig.UC_KEY));
        }

        private string UCApiPost(string module, string action, Hashtable args)
        {
            string data = string.Empty;
            string sep = string.Empty;
            if (args != null)
            {
                this.LoopHashtable(args, ref data, ref sep);
            }
            return this.UCFOpen(UCConfig.UC_API + "index.php", 0, this.UCRequestData(module, action, data, ""), "", true, UCConfig.UC_IP, 0x1388, true);
        }

        private string UCApiUrl(string module, string action, string arg, string extra)
        {
            return (UCConfig.UC_API + "index.php?" + this.UCRequestData(module, action, arg, extra));
        }

        private string UCFOpen(string url, int limit, string post, string cookie, bool bysocket, string ip, int timeout, bool block)
        {
            // Prepare web request

            var encoding = Encoding.GetEncoding(UCConfig.UC_CHARSET);
            byte[] data = encoding.GetBytes(post);

            HttpWebRequest request =
            (HttpWebRequest)WebRequest.Create(url);

            request.AllowAutoRedirect = true;
            request.Accept = "*/*";
            request.UserAgent = this.GetUserAgent();
            request.Headers.Add(HttpRequestHeader.AcceptLanguage, "zh-cn");
            request.Headers.Add(HttpRequestHeader.Cookie, cookie);
            request.Timeout = timeout;
            request.Method = "POST";
            request.ContentType = "application/x-www-form-urlencoded";
            request.ContentLength = data.Length;
            Stream newStream = request.GetRequestStream();

            // Send the data.
            newStream.Write(data, 0, data.Length);
            newStream.Close();




            string str = "";
            using (HttpWebResponse response = (HttpWebResponse) request.GetResponse())
            {
                if (response.StatusCode == HttpStatusCode.OK)
                {
                    using (Stream stream = response.GetResponseStream())
                    {
                        using (StreamReader reader = new StreamReader(stream, Encoding.GetEncoding(UCConfig.UC_CHARSET)))
                        {
                            str = reader.ReadToEnd();
                        }
                    }
                }
            }
            Console.WriteLine(str);
            return str.Trim();
        }

        private string UCRequestData(string module, string action, string arg, string extra)
        {
            string str = this.UCApiInput(arg);
            string str2 = string.Format("m={0}&a={1}&inajax=2&release={2}&input={3}&appid={4}", new object[] { module, action, UCConfig.UC_CLIENT_RELEASE, str, UCConfig.UC_APPID });
            if (string.IsNullOrEmpty(extra))
            {
                return str2;
            }
            if (!extra.StartsWith("&"))
            {
                extra = "&" + extra;
            }
            return (str2 + extra);
        }

    }
}

