namespace FS.API.UCenter.UCClient
{
    using FS.API.UCenter;
    using System;
    using System.Xml;

    public sealed class UC_PM : UCReturnBase
    {
        public int DateLine;
        public short DateRange;
        public short DelStatus;
        public string Folder;
        public int FromAppId;
        public bool IsNew;
        public string Message;
        public string MsgFrom;
        public int MsgFromId;
        public int MsgToId;
        public int PMId;
        public int Related;
        public string Subject;

        public UC_PM()
        {
            this.MsgFrom = "";
            this.Folder = "";
            this.IsNew = false;
            this.Subject = "";
            this.Message = "";
            this.DateRange = 0;
        }

        public UC_PM(string returnString) : base(returnString)
        {
        }

        public UC_PM(XmlNode node) : this()
        {
            this.InitFromXmlNode(node);
        }

        private void InitFromXmlNode(XmlNode node)
        {
            try
            {
                this.PMId = Utils.StringToInt(node.SelectSingleNode("item[@id='pmid']").FirstChild.Value);
                this.MsgFromId = Utils.StringToInt(node.SelectSingleNode("item[@id='msgfromid']").FirstChild.Value);
                this.MsgFrom = node.SelectSingleNode("item[@id='msgfrom']").FirstChild.Value;
                this.Folder = node.SelectSingleNode("item[@id='folder']").FirstChild.Value;
                this.MsgToId = Utils.StringToInt(node.SelectSingleNode("item[@id='msgtoid']").FirstChild.Value);
                this.IsNew = node.SelectSingleNode("item[@id='new']").FirstChild.Value == "1";
                this.Subject = node.SelectSingleNode("item[@id='subject']").FirstChild.Value;
                this.DateLine = Utils.StringToInt(node.SelectSingleNode("item[@id='dateline']").FirstChild.Value);
                this.DelStatus = Utils.StringToShort(node.SelectSingleNode("item[@id='delstatus']").FirstChild.Value);
                this.Related = Utils.StringToInt(node.SelectSingleNode("item[@id='msgtoid']").FirstChild.Value);
                this.FromAppId = Utils.StringToInt(node.SelectSingleNode("item[@id='fromappid']").FirstChild.Value);
                this.DateRange = Utils.StringToShort(node.SelectSingleNode("item[@id='daterange']").FirstChild.Value);
            }
            catch
            {
            }
        }

        public override string ToString()
        {
            return string.Format("PMId={0}, MsgFrom={1}, MsgFromId={2}, IsNew={3}, Subject={4}, DateLine={5}, DateRange={6}", new object[] { this.PMId, this.MsgFrom, this.MsgFromId, this.IsNew, this.Subject, this.DateLine, this.DateRange });
        }

        protected override void UnSerialize()
        {
            XmlDocument document = new XmlDocument();
            try
            {
                document.LoadXml(base.ReturnValue);
                this.InitFromXmlNode(document.DocumentElement);
            }
            catch
            {
            }
        }
    }
}

